<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet
 version="1.0"
 xmlns="http://www.w3.org/2000/svg"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:xalan="http://xml.apache.org/xalan"
 xmlns:xlink="http://www.w3.org/1999/xlink"
 xmlns:math="http://exslt.org/math"
 extension-element-prefixes="math">

<xsl:include href="../../../util/exslt/math.power.template.xsl"/>
<xsl:include href="common-graph.xsl"/>

<xsl:template name="histogram">
	<xsl:param name="rtf-node-set"/>
	<xsl:param name="number-of-blocks"/>
	<xsl:param name="max-in-node-set"/>
  	<xsl:param name="text-x-axis"/>
  	<xsl:param name="text-y-axis"/>
	<xsl:param name=""/>


  	<xsl:variable name="node-set" select="xalan:nodeset($rtf-node-set)"/>

  	<svg width="13cm" height="13cm" viewBox="0 0 1300 1300"
     xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">

	<!-- Include common libraries -->
   	<xsl:call-template name="common-graph">
	  	<xsl:with-param name="text-x-axis" select="$text-x-axis"/>
	  	<xsl:with-param name="text-y-axis" select="$text-y-axis"/>
	</xsl:call-template>


	<xsl:call-template name="histo-boxes-generator">
		<xsl:with-param name="node-set" select="$node-set"/>
		<xsl:with-param name="number-of-blocks" select="$number-of-blocks"/>
		<xsl:with-param name="max-in-node-set" select="$max-in-node-set"/>
		<!-- With the following variable set , the histogram supports values from 0.01 to 10.000.000.000 -->
		<xsl:with-param name="iterations" select="'12'"/>
	</xsl:call-template>
	</svg>
</xsl:template>




<xsl:template name="histo-boxes-generator">
	<xsl:param name="node-set"/>
	<xsl:param name="number-of-blocks"/>
	<xsl:param name="max-in-node-set"/>
	<xsl:param name="iterations"/>

	<xsl:variable name="iteration-ten-power">
		<xsl:call-template name="math:power">
		   <xsl:with-param name="base" select="10" />
		   <xsl:with-param name="power" select="$iterations" />
		</xsl:call-template>
	</xsl:variable>

	<xsl:variable name="min-test-value" select="0.001 * $iteration-ten-power"/>
	<xsl:variable name="max-test-value" select="0.01 * $iteration-ten-power"/>

	<xsl:if test="($max-in-node-set &gt; $min-test-value)and($max-in-node-set &lt;= $max-test-value)">
		<xsl:choose>
			<xsl:when test="$max-in-node-set &lt;= $max-test-value div 5">
				<xsl:call-template name="draw-histo-box-set">
					<xsl:with-param name="node-set" select="$node-set"/>
					<xsl:with-param name="number-of-blocks" select="$number-of-blocks"/>
					<xsl:with-param name="max-x-value" select="$max-test-value div 5"/>
					<xsl:with-param name="iterations" select="$number-of-blocks"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="$max-in-node-set &lt;= $max-test-value div 2">
				<xsl:call-template name="draw-histo-box-set">
					<xsl:with-param name="node-set" select="$node-set"/>
					<xsl:with-param name="number-of-blocks" select="$number-of-blocks"/>
					<xsl:with-param name="max-x-value" select="$max-test-value div 2"/>
					<xsl:with-param name="iterations" select="$number-of-blocks"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="draw-histo-box-set">
					<xsl:with-param name="node-set" select="$node-set"/>
					<xsl:with-param name="number-of-blocks" select="$number-of-blocks"/>
					<xsl:with-param name="max-x-value" select="$max-test-value"/>
					<xsl:with-param name="iterations" select="$number-of-blocks"/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:if>

	<!-- Iterate through recursion -->
	<xsl:if test="$iterations &gt;= 1">
		<xsl:call-template name="histo-boxes-generator">
			<xsl:with-param name="node-set" select="$node-set"/>
			<xsl:with-param name="number-of-blocks" select="$number-of-blocks"/>
			<xsl:with-param name="max-in-node-set" select="$max-in-node-set"/>
			<xsl:with-param name="iterations" select="$iterations - 1"/>
		</xsl:call-template>
	</xsl:if>
</xsl:template>


<xsl:template name="draw-histo-box-set">
	<xsl:param name="node-set"/>
	<xsl:param name="number-of-blocks"/>
	<xsl:param name="max-x-value"/>
	<xsl:param name="max-y-value"/>
	<xsl:param name="iterations"/>

	<xsl:variable name="interval-min">
		<xsl:value-of select="$max-x-value * ($iterations - 1) div $number-of-blocks"/>
	</xsl:variable>

	<xsl:variable name="interval-max">
		<xsl:value-of select="$max-x-value * $iterations div $number-of-blocks"/>
	</xsl:variable>



	<xsl:call-template name="histo-box">
		<xsl:with-param name="y-value" select="count($node-set/points/point[ . &gt;= $interval-min and . &lt;= $interval-max])"/>
		<xsl:with-param name="number-of-blocks" select="$number-of-blocks"/>
		<xsl:with-param name="max-x-value" select="$max-x-value"/>
		<xsl:with-param name="max-y-value" select="count($node-set/points/point/.)"/>
		<xsl:with-param name="block-number" select="$iterations"/>
		<xsl:with-param name="color-ok"		select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 3]/@color"/>
		<xsl:with-param name="color-warn"   select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 2]/@color"/>
		<xsl:with-param name="color-ko"     select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 1]/@color"/>
	</xsl:call-template>


	<!-- Iterate through recursion-->
	<xsl:if test="$iterations &gt; 1">
		<xsl:call-template name="draw-histo-box-set">
			<xsl:with-param name="node-set" select="$node-set"/>
			<xsl:with-param name="number-of-blocks" select="$number-of-blocks"/>
			<xsl:with-param name="max-x-value" select="$max-x-value"/>
			<xsl:with-param name="max-y-value" select="$max-y-value"/>
			<xsl:with-param name="iterations" select="$iterations - 1"/>
		</xsl:call-template>
	</xsl:if>
</xsl:template>


<xsl:template name="histo-box">
	<xsl:param name="y-value"/>
	<xsl:param name="number-of-blocks"/>
	<xsl:param name="max-x-value"/>
	<xsl:param name="max-y-value"/>
	<xsl:param name="block-number"/>
	<xsl:variable name="H" select="1000 * $y-value div $max-y-value"/>
	<xsl:variable name="W" select="1000 div $number-of-blocks"/>

	<rect width="90" stroke="black" stroke-width="2">
		<xsl:attribute name="fill">
			<xsl:value-of select="$color-ok"/>
		</xsl:attribute>
    	<xsl:attribute name="width">
    		<xsl:value-of select="$W"/>
    	</xsl:attribute>
    	<xsl:attribute name="x">
    		<xsl:value-of select="100 + $W * ($block-number - 1)"/>
    	</xsl:attribute>
    	<xsl:attribute name="y">
    		<xsl:value-of select="1200 - $H"/>
    	</xsl:attribute>
    	<xsl:attribute name="height">
    		<xsl:value-of select="$H"/>
    	</xsl:attribute>
    </rect>

	<g>
		<xsl:attribute name="stroke"><xsl:value-of select="$color-ko"/></xsl:attribute>
	    <xsl:if test="$y-value &gt; 0">
		    <text font-family="Verdana" font-size="25" fill="green" >
		    	<xsl:attribute name="x">
	    			<xsl:value-of select="85 + 1000 div $number-of-blocks div 2 + $W * ($block-number - 1)"/>
		    	</xsl:attribute>
		    	<xsl:attribute name="y">
		    		<xsl:value-of select="1190 - $H"/>
		    	</xsl:attribute>
		    	<xsl:value-of select="$y-value"/>
			</text>
		</xsl:if>
		<text font-family="Verdana" font-size="25" fill="green" >
	    	<xsl:attribute name="x">
    			<xsl:value-of select="85 + $W * $block-number"/>
	    	</xsl:attribute>
	    	<xsl:attribute name="y">
	    		<xsl:value-of select="1230"/>
	    	</xsl:attribute>
	    	<xsl:value-of select="format-number($block-number * $max-x-value div $number-of-blocks, '#.##')"/>
		</text>
    </g>

    <g stroke="black" >
    <line x1="100" y1="200" x2="1200" y2="200"
            stroke-dasharray="20 10" stroke-width="2" />
    <text x="150" y="170"
        font-family="Verdana" font-size="25" fill="green" >
    	Total = <xsl:value-of select="$max-y-value"/>
	</text>
   </g>
</xsl:template>

</xsl:stylesheet>
