<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet
 version="1.0"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:redirect="http://xml.apache.org/xalan/redirect"
 extension-element-prefixes="redirect">

   <xsl:output
    method="xml"
    indent="yes"
    encoding="UTF-8" />

    <xsl:param name="system"/>
    <xsl:param name="version"/>
    <xsl:param name="date"/>
    <xsl:param name="radar-config-file"/>
    <xsl:param name="log"/>

    <xsl:variable name="project-config"
    	          select="document($radar-config-file)"/>


   	<xsl:template	match="IntegratedAnalysis">
   		<xsl:choose>
   			<xsl:when test="$log">
				<xsl:message>
system:<xsl:value-of select="$system"/>
version:<xsl:value-of select="$version"/>
date:<xsl:value-of select="$date"/>
radar-config:<xsl:value-of select="$radar-config-file"/>
system from radar-config:<xsl:value-of select="$project-config/radar-config/attribute::name"/>
   				</xsl:message>
   			</xsl:when>
   		</xsl:choose>

      <xsl:copy>
      	 <xsl:attribute name="system">
      	 	<xsl:value-of select="$system"/>
      	 </xsl:attribute>
      	 <xsl:attribute name="version">
      	 	<xsl:value-of select="$version"/>
      	 </xsl:attribute>
      	 <xsl:attribute name="date">
      	 	<xsl:value-of select="$date"/>
      	 </xsl:attribute>
      	 <xsl:copy-of select="source"/>

         <Packages>
         	<xsl:for-each select="Packages/Package">
         		<xsl:sort select="@name"/>
         		<xsl:variable name="package-name" select="@name"/>

                <xsl:choose>
                <xsl:when test="count($project-config/radar-config/skip-packages/package-root[(@recurse = 'true' and starts-with($package-name,@value)) or (@recurse = 'false' and $package-name = @value)]) = 0">
                <xsl:variable name="this-subsystem-id">
					<xsl:choose>
						<xsl:when test="$project-config/radar-config/subsystems/subsystem/included-packages/package-root[(starts-with($package-name,@value)) and (@recurse='true')]">
							<xsl:for-each select="$project-config/radar-config/subsystems/subsystem/included-packages/package-root[(starts-with($package-name,@value)) and (@recurse='true')]">
							<xsl:value-of select="ancestor::subsystem/@id"/>
							</xsl:for-each>
						</xsl:when>
						<xsl:otherwise>
							<xsl:for-each select="$project-config/radar-config/subsystems/subsystem/included-packages/package-root[($package-name = @value) and (@recurse='false')]">
							<xsl:value-of select="ancestor::subsystem/@id"/>
							</xsl:for-each>
						 </xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
         		<xsl:copy>
         			<xsl:apply-templates select="@*"/>
					<xsl:if test="(count($project-config/radar-config/system-api/package-root[(@recurse='false') and (@value=$package-name)]) &gt; 0) or
						(count($project-config/radar-config/system-api/package-root[(@recurse='true') and (starts-with($package-name,@value))]) &gt; 0)">
						<xsl:attribute name="public-api-package">true</xsl:attribute>
					</xsl:if>
					<subsystem>
						<xsl:attribute name="name">
							<xsl:value-of select="normalize-space($project-config/radar-config/subsystems/subsystem[@id = $this-subsystem-id]/name)"/>
						</xsl:attribute>
						<xsl:attribute name="id">
							<xsl:value-of select="$this-subsystem-id"/>
						</xsl:attribute>
					</subsystem>
         			<xsl:apply-templates select="node()">
         				<xsl:with-param name="this-subsystem-id" select="$this-subsystem-id"/>
         			</xsl:apply-templates>
         		</xsl:copy>
                </xsl:when>
                <xsl:otherwise>
                    <Package name="{$package-name}">
                      <error>Package was skipped analysis.</error>
                    </Package>
                </xsl:otherwise>
                </xsl:choose>
         	</xsl:for-each>
         </Packages>
      </xsl:copy>
   </xsl:template>

   <xsl:template
    match="Stats">
       <xsl:param name="this-subsystem-id"/>
	   <xsl:copy>
         <xsl:apply-templates select="@*|node()"/>

		 <xsl:if test="not(test-metrics)">
			<xsl:variable name="npm" select="sum(../descendant::Class/oo-metrics/npm)"/>
			<xsl:variable name="methods-referenced" select="sum(../descendant::Class/test-metrics/methods-referenced)"/>
			<test-metrics>
				<methods-referenced><xsl:value-of select="$methods-referenced"/></methods-referenced>
				<method-coverage><xsl:value-of select="$methods-referenced div $npm"/></method-coverage>
			</test-metrics>
		</xsl:if>

         <cycles><xsl:value-of
         			select="../Cycles/@TotalNumber &gt; 0"/></cycles>

         <xsl:call-template name="java-metrics">
			<xsl:with-param name="code-violations" select="count(../AbstractClasses/Class/violations/violation)+count(../ConcreteClasses/Class/violations/violation)"/>
			<xsl:with-param name="style-errors" select="count(../AbstractClasses/Class/style-errors/error)+count(../ConcreteClasses/Class/style-errors/error)"/>
			<xsl:with-param name="classes-with-code-violations" select="count(../AbstractClasses/Class[violations/violation])+count(../ConcreteClasses/Class[violations/violation])"/>
			<xsl:with-param name="classes-with-style-errors" select="count(../AbstractClasses/Class[style-errors/error])+count(../ConcreteClasses/Class[style-errors/error])"/>

            <xsl:with-param name="business-critical-classes">
                <xsl:choose>
                    <xsl:when test="count(/IntegrattedAnalysis/source[normalize-space(.) = 'BCC']) = 1"><xsl:value-of select="count(../AbstractClasses/Class/business-critical)+count(../ConcreteClasses/Class/business-critical)"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="TotalClasses"/></xsl:otherwise>
                </xsl:choose>
            </xsl:with-param>
			<xsl:with-param name="source-classes-analyzed" select="count(../AbstractClasses/Class[analysed-in-detail])+count(../ConcreteClasses/Class[analysed-in-detail])"/>
			<xsl:with-param name="total-ccn-for-package" select="sum(../AbstractClasses/Class/functions/function/ccn)+sum(../ConcreteClasses/Class/functions/function/ccn)"/>
		 </xsl:call-template>

		 <xsl:apply-templates select=".." mode="oo-metrics"/>

         <xsl:apply-templates select=".." mode="duplication-metrics"/>

         <xsl:apply-templates select=".." mode="dependancy-metrics">
			<xsl:with-param name="this-subsystem-id" select="$this-subsystem-id"/>
         </xsl:apply-templates>

      </xsl:copy>
   </xsl:template>

   <xsl:template name="java-metrics">
		<xsl:param name="code-violations"/>
		<xsl:param name="classes-with-code-violations"/>
		<xsl:param name="style-errors"/>
		<xsl:param name="classes-with-style-errors"/>
		<xsl:param name="business-critical-classes"/>
		<xsl:param name="source-classes-analyzed"/>
		<xsl:param name="total-ccn-for-package"/>

		<code-violations>
		<xsl:value-of select="$code-violations"/>
		</code-violations>
		<code-violations-per-class>
		<xsl:value-of select="$code-violations div $source-classes-analyzed"/>
		</code-violations-per-class>
        <classes-with-code-violations>
		<xsl:value-of select="$classes-with-code-violations"/>
		</classes-with-code-violations>

        <style-errors>
		<xsl:value-of select="$style-errors"/>
		</style-errors>
		<style-errors-per-class>
		<xsl:value-of select="$style-errors div $source-classes-analyzed"/>
		</style-errors-per-class>
		<classes-with-style-errors>
		<xsl:value-of select="$classes-with-style-errors"/>
		</classes-with-style-errors>

        <business-critical-classes>
		<xsl:value-of select="$business-critical-classes"/>
		</business-critical-classes>
		<ccn>
		<xsl:value-of select="$total-ccn-for-package"/>
		</ccn>
		<source-classes-analyzed>
		<xsl:value-of select="$source-classes-analyzed"/>
		</source-classes-analyzed>
   	</xsl:template>

	<xsl:template match="Package" mode="oo-metrics">
   		<xsl:variable name="package-element" select="current()"/>
            <oo-metrics>
                <wmc><xsl:value-of select="sum(descendant::oo-metrics/wmc) div Stats/TotalClasses"/></wmc>
                <dit><xsl:value-of select="sum(descendant::oo-metrics/dit) div Stats/TotalClasses"/></dit>
                <noc><xsl:value-of select="sum(descendant::oo-metrics/noc) div Stats/TotalClasses"/></noc>
                <cbo><xsl:value-of select="sum(descendant::oo-metrics/cbo) div Stats/TotalClasses"/></cbo>
                <rfc><xsl:value-of select="sum(descendant::oo-metrics/rfc) div Stats/TotalClasses"/></rfc>
                <lcom><xsl:value-of select="sum(descendant::oo-metrics/lcom) div Stats/TotalClasses"/></lcom>
                <ca><xsl:value-of select="sum(descendant::oo-metrics/ca) div Stats/TotalClasses"/></ca>
                <npm><xsl:value-of select="sum(descendant::oo-metrics/npm)"/></npm>
            </oo-metrics>
   	</xsl:template>

    <xsl:template match="Package" mode="duplication-metrics">
   		<xsl:variable name="package-element" select="current()"/>

        <xsl:if test="sum(descendant::duplications/@total-number) &gt; 0">
            <duplication-metrics>
                <duplications><xsl:value-of select="sum(descendant::duplications/@total-number)"/></duplications>
                <lines><xsl:value-of select="sum(descendant::duplication/descendant::lines)"/></lines>
                <tokens><xsl:value-of select="sum(descendant::duplication/descendant::tokens)"/></tokens>
                <duplicated-tokens-per-ncss><xsl:value-of select="sum(descendant::duplication/descendant::tokens) div Stats/ncss"/></duplicated-tokens-per-ncss>
                <classes-with-duplications><xsl:value-of select="count(AbstractClasses/Class[duplications/@total-number &gt; 0]) + count(ConcreteClasses/Class[duplications/@total-number &gt; 0])"/></classes-with-duplications>
            </duplication-metrics>
        </xsl:if>
   	</xsl:template>

	<xsl:template match="coupling-metrics">
		<xsl:copy>
			<xsl:apply-templates select="@*|node()"/>
			<xsl:choose>
				<xsl:when test="count(../../Stats) &gt; 0">
					<class-internal-imports><xsl:value-of select="count(../../descendant::functions/function/coupling/import[@class-internal = 'true'])"/></class-internal-imports>
					<package-internal-imports><xsl:value-of select="count(../../descendant::functions/function/coupling/import[@package-internal = 'true'])"/></package-internal-imports>
					<subsystem-internal-imports><xsl:value-of select="count(../../descendant::functions/function/coupling/import[@subsystem-internal = 'true'])"/></subsystem-internal-imports>
					<system-internal-imports><xsl:value-of select="count(../../descendant::functions/function/coupling/import[@system-internal = 'true'])"/></system-internal-imports>
					<external-imports><xsl:value-of select="count(../../descendant::functions/function/coupling/import[not(@system-internal = 'true')])"/></external-imports>
					<runtime-library-imports><xsl:value-of select="count(../../descendant::functions/function/coupling/import[@runtime-library = 'true'])"/></runtime-library-imports>
					<class-internal-exports><xsl:value-of select="count(../../descendant::functions/function/coupling/export[@class-internal = 'true'])"/></class-internal-exports>
					<package-internal-exports><xsl:value-of select="count(../../descendant::functions/function/coupling/export[@package-internal = 'true'])"/></package-internal-exports>
					<subsystem-internal-exports><xsl:value-of select="count(../../descendant::functions/function/coupling/export[@subsystem-internal = 'true'])"/></subsystem-internal-exports>
					<system-internal-exports><xsl:value-of select="count(../../descendant::functions/function/coupling/export[@system-internal = 'true'])"/></system-internal-exports>
				</xsl:when>
				<xsl:otherwise>
					<class-internal-imports><xsl:value-of select="count(../descendant::functions/function/coupling/import[@class-internal = 'true'])"/></class-internal-imports>
					<package-internal-imports><xsl:value-of select="count(../descendant::functions/function/coupling/import[@package-internal = 'true'])"/></package-internal-imports>
					<subsystem-internal-imports><xsl:value-of select="count(../descendant::functions/function/coupling/import[@subsystem-internal = 'true'])"/></subsystem-internal-imports>
					<system-internal-imports><xsl:value-of select="count(../descendant::functions/function/coupling/import[@system-internal = 'true'])"/></system-internal-imports>
					<external-imports><xsl:value-of select="count(../descendant::functions/function/coupling/import[not(@system-internal = 'true')])"/></external-imports>
					<runtime-library-imports><xsl:value-of select="count(../descendant::functions/function/coupling/import[@runtime-library = 'true'])"/></runtime-library-imports>
					<class-internal-exports><xsl:value-of select="count(../descendant::functions/function/coupling/export[@class-internal = 'true'])"/></class-internal-exports>
					<package-internal-exports><xsl:value-of select="count(../descendant::functions/function/coupling/export[@package-internal = 'true'])"/></package-internal-exports>
					<subsystem-internal-exports><xsl:value-of select="count(../descendant::functions/function/coupling/export[@subsystem-internal = 'true'])"/></subsystem-internal-exports>
					<system-internal-exports><xsl:value-of select="count(../descendant::functions/function/coupling/export[@system-internal = 'true'])"/></system-internal-exports>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:copy>
	</xsl:template>

   	<xsl:template match="Package" mode="dependancy-metrics">
   		<xsl:param name="this-subsystem-id"/>
		<xsl:variable name="package-element" select="current()"/>
   		<xsl:variable name="packages-element" select=".."/>
   		<xsl:variable name="package-name" select="@name"/>

   		<dependancy-metrics>
   			<xsl:for-each select="$project-config/radar-config/subsystems/subsystem">
   				<xsl:variable name="used-subsystem-id" select="@id"/>
   				<subsystem>
	   				<xsl:attribute name="name">
	   					<xsl:value-of select="normalize-space(name)"/>
	   				</xsl:attribute>
	   				<xsl:attribute name="legal">
	   					<xsl:choose>
	   						<xsl:when test="count($project-config/radar-config/subsystems/subsystem[@id=$this-subsystem-id]/legal-subordinates/subsystem[@id=normalize-space($used-subsystem-id)]) &gt; 0">true</xsl:when>
	   						<xsl:otherwise>false</xsl:otherwise>
	   					</xsl:choose>
	   				</xsl:attribute>
	   				<xsl:call-template name="find-subsystem-dependancy-in-package">
						<xsl:with-param name="package-element" select="$package-element"/>
						<xsl:with-param name="included-packages" select="included-packages"/>
				   		<xsl:with-param name="position" select="1"/>
						<xsl:with-param name="dependancy-count" select="0"/>
					</xsl:call-template>
   				</subsystem>
   			</xsl:for-each>

			<subsystem name="External">
				<xsl:call-template name="find-external-dependancy-in-package">
					<xsl:with-param name="packages-element" select="$packages-element"/>
					<xsl:with-param name="package-element" select="$package-element"/>
			   		<xsl:with-param name="position" select="1"/>
					<xsl:with-param name="dependancy-count" select="0"/>
				</xsl:call-template>
			</subsystem>

   		</dependancy-metrics>
   	</xsl:template>

   	<xsl:template name="find-subsystem-dependancy-in-package">
   		<xsl:param name="package-element"/>
   		<xsl:param name="included-packages"/>
   		<xsl:param name="position"/>
		<xsl:param name="dependancy-count"/>

		<xsl:choose>
			<xsl:when test="$position &lt; count($included-packages/package-root)">
				<xsl:call-template name="find-subsystem-dependancy-in-package">
					<xsl:with-param name="package-element" select="$package-element"/>
					<xsl:with-param name="included-packages" select="$included-packages"/>
			   		<xsl:with-param name="position" select="$position + 1"/>
					<xsl:with-param name="dependancy-count"
						select="$dependancy-count
								+ count($package-element/DependsUpon/Package[starts-with(normalize-space(node()),
									$included-packages/package-root[position() = $position]/@value)])"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of
						select="$dependancy-count
								+ count($package-element/DependsUpon/Package[starts-with(normalize-space(node()),
									$included-packages/package-root[position() = $position]/@value)])"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

   	<xsl:template name="find-external-dependancy-in-package">
   		<xsl:param name="packages-element"/>
   		<xsl:param name="package-element"/>
   		<xsl:param name="position"/>
		<xsl:param name="dependancy-count"/>

		<xsl:variable name="dependancy-package-name" select="normalize-space($package-element/DependsUpon/Package[position() = $position]/node())"/>
		<xsl:choose>
			<xsl:when test="$position &lt; count($package-element/DependsUpon/Package)">
				<xsl:call-template name="find-external-dependancy-in-package">
					<xsl:with-param name="packages-element" select="$packages-element"/>
					<xsl:with-param name="package-element" select="$package-element"/>
			   		<xsl:with-param name="position" select="$position + 1"/>
					<xsl:with-param name="dependancy-count"
						select="$dependancy-count
								+ count($packages-element/Package[(count(error) &gt; 0) and (not(starts-with(@name,'java.'))) and (@name=$dependancy-package-name)])"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of
						select="$dependancy-count
								+ count($packages-element/Package[(count(error) &gt; 0) and (not(starts-with(@name,'java.'))) and (@name=$dependancy-package-name)])"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

   <xsl:template
    match="UsedBy">
		<xsl:param name="this-subsystem-id"/>
    	<xsl:variable name="this-package-name" select="../@name"/>

    	<xsl:copy>
    		<xsl:for-each select="Package">
    			<xsl:variable name="depending-package-name" select="normalize-space(node())"/>
    			<!--xsl:variable name="depending-subsystem-id" select="$project-config/radar-config/subsystems/subsystem/included-packages/package-root[starts-with($depending-package-name,@value)]/ancestor::subsystem/@id"/-->
    			<xsl:variable name="depending-subsystem-id">
    				<xsl:choose>
    					<xsl:when test="$project-config/radar-config/subsystems/subsystem/included-packages/package-root[(starts-with($depending-package-name,@value)) and (@recurse='true')]">
    						<xsl:value-of select="$project-config/radar-config/subsystems/subsystem/included-packages/package-root[(starts-with($depending-package-name,@value)) and (@recurse='true')]/ancestor::subsystem/@id"/>
    					</xsl:when>
    					<xsl:otherwise>
    						<xsl:value-of select="$project-config/radar-config/subsystems/subsystem/included-packages/package-root[($depending-package-name = @value) and (@recurse='false')]/ancestor::subsystem/@id"/>
    					</xsl:otherwise>
    				</xsl:choose>
    			</xsl:variable>
    			<xsl:copy>
	    			<xsl:choose>
	    				<xsl:when test="count($project-config/radar-config/subsystems/subsystem[@id=normalize-space($depending-subsystem-id)]/legal-subordinates/subsystem[@id=normalize-space($this-subsystem-id)]) &gt; 0">
	    					<xsl:attribute name="legal">true</xsl:attribute>
	    				</xsl:when>
	    				<xsl:otherwise>
	    					<xsl:attribute name="legal">false</xsl:attribute>
	    				</xsl:otherwise>
	    			</xsl:choose>
	    			<xsl:attribute name="subsystem-name"><xsl:value-of select="normalize-space($project-config/radar-config/subsystems/subsystem[@id = normalize-space($depending-subsystem-id)]/name)"/></xsl:attribute>
	    			<xsl:apply-templates select="node()"/>
	    		</xsl:copy>
    		</xsl:for-each>
    	</xsl:copy>
   </xsl:template>

   <xsl:template
    match="DependsUpon">
    	<xsl:param name="this-subsystem-id"/>
		<xsl:variable name="this-package-name" select="../@name"/>


    	<xsl:copy>
    		<xsl:for-each select="Package">
    			<xsl:variable name="used-package-name" select="normalize-space(node())"/>
    			<xsl:variable name="used-subsystem-id">
    				<xsl:choose>
    					<xsl:when test="$project-config/radar-config/subsystems/subsystem/included-packages/package-root[(starts-with($used-package-name,@value)) and (@recurse='true')]">
    						<xsl:value-of select="$project-config/radar-config/subsystems/subsystem/included-packages/package-root[(starts-with($used-package-name,@value)) and (@recurse='true')]/ancestor::subsystem/@id"/>
    					</xsl:when>
    					<xsl:otherwise>
    						<xsl:value-of select="$project-config/radar-config/subsystems/subsystem/included-packages/package-root[($used-package-name = @value) and (@recurse='false')]/ancestor::subsystem/@id"/>
    					</xsl:otherwise>
    				</xsl:choose>
    			</xsl:variable>
    			<xsl:copy>
	    			<xsl:choose>
	    				<xsl:when test="count($project-config/radar-config/subsystems/subsystem[@id=normalize-space($this-subsystem-id)]/legal-subordinates/subsystem[@id=normalize-space($used-subsystem-id)]) &gt; 0">
	    					<xsl:attribute name="legal">true</xsl:attribute>
	    				</xsl:when>
	    				<xsl:otherwise>
	    					<xsl:attribute name="legal">false</xsl:attribute>
	    				</xsl:otherwise>
	    			</xsl:choose>
	    			<xsl:if test="../../../Package[@name=$used-package-name]/error">
	    				<xsl:attribute name="external">true</xsl:attribute>
	    			</xsl:if>
	    			<xsl:attribute name="subsystem-name"><xsl:value-of select="normalize-space($project-config/radar-config/subsystems/subsystem[@id = normalize-space($used-subsystem-id)]/name)"/></xsl:attribute>
	    			<xsl:apply-templates select="node()"/>
	    		</xsl:copy>
    		</xsl:for-each>
    	</xsl:copy>
   </xsl:template>

    <xsl:template match="Class">
		<xsl:param name="this-subsystem-id"/>
        <xsl:copy>
            <xsl:apply-templates select="@*|node()">
				<xsl:with-param name="this-subsystem-id" select="$this-subsystem-id"/>
			</xsl:apply-templates>
            <xsl:if test="count(/IntegratedAnalysis/source[normalize-space(.) = 'BCC']) &lt; 1"><business-critical/></xsl:if>
        </xsl:copy>
    </xsl:template>

   <xsl:template
    match="import|export">
   		<xsl:param name="this-subsystem-id"/>
   		<xsl:copy>
   			<xsl:apply-templates select="@*"/>
			<xsl:if test="not(@subsystem-internal = 'true') and not(@runtime-library = 'true')">
				<xsl:choose>
					<xsl:when test="not(@subsystem-id)">
						<xsl:attribute name="system-internal">false</xsl:attribute>
						<xsl:attribute name="legal">true</xsl:attribute>
					</xsl:when>
					<xsl:when test="@subsystem-id = $this-subsystem-id">
						<xsl:attribute name="subsystem-internal">true</xsl:attribute>
						<xsl:attribute name="legal">true</xsl:attribute>
					</xsl:when>
					<xsl:otherwise>
						<xsl:variable name="used-subsystem-id" select="@subsystem-id"/>
						<xsl:attribute name="legal">
							<xsl:choose>
								<xsl:when
									test="name() = 'import' and
										count($project-config/radar-config/subsystems/subsystem[@id=$this-subsystem-id]/legal-subordinates/subsystem[@id=normalize-space($used-subsystem-id)]) &gt; 0">true</xsl:when>
								<xsl:when
									test="name() = 'export' and
										count($project-config/radar-config/subsystems/subsystem[@id=$used-subsystem-id]/legal-subordinates/subsystem[@id=normalize-space($this-subsystem-id)]) &gt; 0">true</xsl:when>
								<xsl:otherwise>false</xsl:otherwise>
							</xsl:choose>
						</xsl:attribute>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:if>
   			<xsl:apply-templates select="node()"/>
   		</xsl:copy>
   </xsl:template>

   <xsl:template
    match="standard-metrics">
   		<xsl:variable name="total-ccn-for-class" select="sum(../functions/function/ccn)"/>
   		<xsl:copy>
   			<xsl:apply-templates select="@*|node()"/>
	   		<ccn>
	         	<xsl:value-of select="$total-ccn-for-class"/>
	        </ccn>
   		</xsl:copy>
   </xsl:template>

   <xsl:template
    match="test-metrics">
   		<xsl:copy>
   			<xsl:apply-templates select="@*|node()"/>
            <xsl:if	test="(ancestor::function) and (line-coverage)">
   				<source-statements-covered><xsl:value-of select="line-coverage * ../ncss"/></source-statements-covered>
   			</xsl:if>
			<xsl:variable name="npm" select="sum(../../descendant::Class/oo-metrics/npm)"/>
			<xsl:if test="ancestor::Stats">
				<xsl:variable name="methods-referenced" select="sum(../../descendant::Class/test-metrics/methods-referenced)"/>
				<methods-referenced><xsl:value-of select="$methods-referenced"/></methods-referenced>
				<method-coverage><xsl:value-of select="$methods-referenced div $npm"/></method-coverage>
			</xsl:if>
   		</xsl:copy>
   </xsl:template>


   <xsl:template name="package-addition"/>

   	<!-- Copy all nodes from here. Can be overridden by a more specific XPath -->
	<xsl:template match="@*|node()">
		<xsl:param name="this-subsystem-id"/>
		<xsl:copy>
			<xsl:apply-templates select="@*|node()">
				<xsl:with-param name="this-subsystem-id" select="$this-subsystem-id"/>
			</xsl:apply-templates>
		</xsl:copy>
	</xsl:template>


</xsl:stylesheet>
