<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet
		version="1.0"
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

	<xsl:include href="xradar://stylesheets/system/radar/process/normalize-report-system-specific.xsl"/>

   <xsl:output method="xml" indent="yes" encoding="UTF-8" />


   	<xsl:template match="IntegratedAnalysis">
      	<radar-report>
      	 	<xsl:apply-templates select="@*"/>
      	 	<sources>
      	 		<xsl:copy-of select="source"/>
         	</sources>
         	<packages>
         		<xsl:apply-templates select="Packages/Package[Stats]"/>
         	</packages>
      	</radar-report>
   	</xsl:template>

   	<xsl:template match="Package">
      	<!--xsl:sort select="@name"/-->
 		<package>
 			<xsl:attribute name="subsystem"><xsl:value-of select="subsystem/@name"/></xsl:attribute>
 			<xsl:apply-templates select="@*"/>
 			<xsl:apply-templates select="Stats"/>
			<xsl:apply-templates select="DependsUpon"/>
 			<classes>
 				<xsl:apply-templates select="AbstractClasses/Class|ConcreteClasses/Class"/>
 			</classes>
 		</package>
   	</xsl:template>

   	<xsl:template match="Stats">
      	<statistics>
      		<xsl:if test="TotalClasses|ConcreteClasses|AbstractClasses|functions|ncss|ccn|code-violations|style-errors">
	      		<source-metrics>
	         		<xsl:apply-templates select="TotalClasses|source-classes-analyzed|ConcreteClasses|AbstractClasses|functions|ncss|ccn|code-violations|style-errors|classes-with-code-violations|classes-with-style-errors"/>
	         	</source-metrics>
	        </xsl:if>
	        <xsl:if test="Ca|Ce|A|I|D|cycles">
	         	<package-oo-metrics>
	         		<xsl:apply-templates select="Ca|Ce|A|I|D|cycles"/>
	         	</package-oo-metrics>
         	</xsl:if>
         	<xsl:if test="javadocs|javadoc_lines|single_comment_lines|multi_comment_lines">
	         	<documentation-metrics>
	         		<xsl:apply-templates select="javadocs|javadoc_lines|single_comment_lines|multi_comment_lines"/>
	         	</documentation-metrics>
         	</xsl:if>
         	<xsl:apply-templates select="duplication-metrics"/>
			<xsl:apply-templates select="coupling-metrics"/>
	        <xsl:apply-templates select="dependancy-metrics"/>
            <xsl:apply-templates select="test-metrics"/>
         	<xsl:apply-templates select="sql-metrics"/>
            <xsl:apply-templates select="source-control-metrics"/>
            <xsl:apply-templates select="oo-metrics"/>
      	</statistics>
   	</xsl:template>

   	<!-- Conversion of source-metrics-->
   	<xsl:template match="TotalClasses">
      	<total-classes><xsl:value-of select="."/></total-classes>
   	</xsl:template>
    <xsl:template match="source-classes-analyzed">
      	<analysed-classes><xsl:value-of select="."/></analysed-classes>
   	</xsl:template>
   	<xsl:template match="ConcreteClasses">
      	<concrete-classes><xsl:value-of select="."/></concrete-classes>
   	</xsl:template>
   	<xsl:template match="AbstractClasses">
      	<abstract-classes><xsl:value-of select="."/></abstract-classes>
   	</xsl:template>

   	<!-- Conversion of package-oo-metrics-->
   	<xsl:template match="Ca">
      	<afferent-coupling><xsl:value-of select="."/></afferent-coupling>
   	</xsl:template>
   	<xsl:template match="Ce">
      	<efferent-coupling><xsl:value-of select="."/></efferent-coupling>
   	</xsl:template>
   	<xsl:template match="A">
      	<abstractness><xsl:value-of select="."/></abstractness>
   	</xsl:template>
   	<xsl:template match="I">
      	<instability><xsl:value-of select="."/></instability>
   	</xsl:template>
   	<xsl:template match="D">
      	<distance><xsl:value-of select="."/></distance>
   	</xsl:template>

   	<!-- Conversion of documentation-metrics-->
   	<xsl:template match="javadoc_lines">
      	<javadoc-lines><xsl:value-of select="."/></javadoc-lines>
   	</xsl:template>
   	<xsl:template match="single_comment_lines">
      	<single-comment-lines><xsl:value-of select="."/></single-comment-lines>
   	</xsl:template>
   	<xsl:template match="multi_comment_lines">
      	<multi-comment-lines><xsl:value-of select="."/></multi-comment-lines>
   	</xsl:template>

    <xsl:template match="dependancy-metrics">
      	 <xsl:copy>
            <legal-dependencies><xsl:value-of select="sum(subsystem[@legal='true'])"/></legal-dependencies>
            <illegal-dependencies><xsl:value-of select="sum(subsystem[@legal!='true'])"/></illegal-dependencies>
	        <xsl:apply-templates select="@*|node()"/>
	     </xsl:copy>
   	</xsl:template>

	<xsl:template match="DependsUpon">
      	<package-dependencies>
      		<xsl:apply-templates select="Package" mode="package-dependencies"/>
      	</package-dependencies>
   	</xsl:template>

	<xsl:template match="Package" mode="package-dependencies">
 		<package>
 			<xsl:apply-templates select="@*|node()"/>
 		</package>
   	</xsl:template>

   	<xsl:template match="Class">
	      	<class>
	      		<xsl:attribute name="name"><xsl:value-of select="substring-before(@sourceFile,'.java')"/></xsl:attribute>
	      		<xsl:attribute name="abstract"><xsl:choose><xsl:when test="name(../node())='AbstractClasses'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose></xsl:attribute>

	      		<statistics>
                      <source-metrics>
                        <xsl:apply-templates select="standard-metrics/ncss|standard-metrics/ccn"/>
                        <methods><xsl:value-of select="sum(standard-metrics/functions)"/></methods>
                        <violations><xsl:value-of select="sum(violations/@total-number)"/></violations>
                        <style-errors><xsl:value-of select="sum(style-errors/@total-number)"/></style-errors>
                      </source-metrics>
                      <documentation-metrics>
                        <xsl:apply-templates select="standard-metrics/javadocs"/>
                      </documentation-metrics>
                      <xsl:apply-templates select="coupling-metrics"/>
					  <xsl:apply-templates select="test-metrics"/>
					  <xsl:apply-templates select="oo-metrics"/>
                      <duplication-metrics>
                        <duplications><xsl:value-of select="sum(duplications/@total-number)"/></duplications>
                        <lines><xsl:value-of select="sum(duplications/duplication/lines)"/></lines>
                        <tokens><xsl:value-of select="sum(duplications/duplication/tokens)"/></tokens>
                        <duplicated-tokens-per-ncss><xsl:value-of select="sum(duplications/duplication/tokens) div sum(standard-metrics/ncss)"/></duplicated-tokens-per-ncss>
                      </duplication-metrics>

	      		</statistics>

                <methods>
	      			<xsl:apply-templates select="functions/function">
	      				<xsl:with-param name="package-class-name" select="normalize-space(node())"/>
	      			</xsl:apply-templates>
	      		</methods>

                <xsl:apply-templates select="duplications"/>
                <xsl:apply-templates select="violations"/>
                <xsl:apply-templates select="style-errors"/>
                <xsl:apply-templates select="source-control-metrics"/>
              </class>
   	</xsl:template>


   	<xsl:template match="function">
   		<xsl:param name="package-class-name"/>
   		<method>
   			<xsl:attribute name="name"><xsl:value-of select="substring-after(name,concat($package-class-name,'.'))"/></xsl:attribute>
	   		<statistics>
	   			<xsl:apply-templates select="ncss|ccn|test-metrics/line-coverage|test-metrics/source-statements-covered|coupling-metrics/import-coupling|coupling-metrics/export-coupling"/>
	   		</statistics>
			<xsl:apply-templates select="coupling"/>
	   	</method>
   	</xsl:template>

   	<!-- Copy all nodes from here. Can be overridden by a more specific XPath -->
	<xsl:template match="@*|node()">
	   <xsl:copy>
	      <xsl:apply-templates select="@*|node()"/>
	   </xsl:copy>
	</xsl:template>



</xsl:stylesheet>
