<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0"
                xmlns:lxslt="http://xml.apache.org/xslt"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:redirect="http://xml.apache.org/xalan/redirect"
                extension-element-prefixes="redirect">

  <xsl:output method="html" encoding="UTF-8"/>

  <xsl:param name="log"/>
  <xsl:param name="docs-home"/>
  <xsl:param name="radar-config-file"/>
  <xsl:param name="java-doc-configured"/>
  <xsl:param name="java-doc-root"/>
  <xsl:param name="dynamics-configured"/>
  <xsl:param name="dynamics-root"/>
  <xsl:param name="jcoverage-configured"/>
  <xsl:param name="jcoverage-root"/>
  <xsl:param name="cobertura-configured"/>
  <xsl:param name="cobertura-root"/>
  
  <xsl:variable name="radar-config-doc"
                select="document($radar-config-file)"/>

  <xsl:variable name="metric-descriptions-doc"
                select="'xradar://stylesheets/etc/data/metric-descriptions.xml'"/>

  <xsl:include href="xradar://stylesheets/radar/style/style.xsl"/> 
  <xsl:include href="xradar://stylesheets/radar/statics/report/framework/master-report-common.xsl"/>
  <xsl:include href="xradar://stylesheets/radar/statics/report/framework/master-report-common-link.xsl"/>
  <xsl:include href="xradar://stylesheets/radar/statics/report/framework/master-report-variables.xsl"/>
  <xsl:include href="xradar://stylesheets/radar/statics/report/scorecard/master-report-detail-scorecard.xsl"/>
  <xsl:include href="xradar://stylesheets/radar/statics/report/master-report-detail-subsystems.xsl"/>
  <xsl:include href="xradar://stylesheets/radar/statics/report/master-report-detail-packages.xsl"/>
  <xsl:include href="xradar://stylesheets/radar/statics/report/analysis/master-report-detail-analysis.xsl"/>

<!-- 
 Sample stylesheet to be used with Analysis XML output.
 It creates a set of HTML files a la javadoc where you can browse easily
 through all packages and classes.
  -->
<xsl:template match="IntegratedAnalysis">

  <xsl:choose>
    <xsl:when test="$log = 'true'">
      <xsl:message>
***************************************************************************************************
log:<xsl:value-of select="$log"/>
docs-home:<xsl:value-of select="$docs-home"/><!-- 
radar-config-doc:<xsl:value-of select="$radar-config-doc/radar-config/attribute::name"/>  -->
radar-config (filename) :<xsl:value-of select="$radar-config-file"/>
java-doc-configured:<xsl:value-of select="$java-doc-configured"/>
java-doc-root:<xsl:value-of select="$java-doc-root"/>
dynamics-configured:<xsl:value-of select="$dynamics-configured"/>
dynamics-root:<xsl:value-of select="$dynamics-root"/>
jcoverage-configured:<xsl:value-of select="$jcoverage-configured"/>
jcoverage-root:<xsl:value-of select="$jcoverage-root"/>
cobertura-configured:<xsl:value-of select="$cobertura-configured"/>
cobertura-root:<xsl:value-of select="$cobertura-root"/>
input-empty ?:<xsl:value-of select="@system"/>
metric-descriptions-doc:<xsl:value-of select="$metric-descriptions-doc"/>
ROOT:<xsl:value-of select="name(*)"/>
***************************************************************************************************
      </xsl:message>
    </xsl:when>
  </xsl:choose>

    <xsl:variable name="packages-without-subsystem" select="count(Packages/Package[(not(subsystem) or normalize-space(subsystem/@name)='') and not(error)])"/>
    
     <!-- create the index.html -->
     <xsl:call-template name="log-step">
     	<xsl:with-param name="log" select="$log"/>
     	<xsl:with-param name="step" select="'Generating index.html'"/>
     </xsl:call-template>
     <redirect:write file="{$docs-home}/index.html">
        <xsl:call-template name="index.html"/>
     </redirect:write>

    <!-- create the stylesheet.css  --> 
    <xsl:call-template name="log-step">
     	<xsl:with-param name="log" select="$log"/>
     	<xsl:with-param name="step" select="'Generating style.css'"/>
    </xsl:call-template>
    <redirect:write file="{$docs-home}/stylesheet.css">
      <xsl:call-template name="stylesheet.css">
        	<xsl:with-param name="radar-config" select="$radar-config-doc"/>
      </xsl:call-template>
     </redirect:write>
     
    <!-- create the all-subsystems.html at the root  --> 
    <xsl:call-template name="log-step">
     	<xsl:with-param name="log" select="$log"/>
     	<xsl:with-param name="step" select="'Generating all-subsystems.html'"/>
    </xsl:call-template>
    <redirect:write file="{$docs-home}/all-subsystems.html">
      <xsl:apply-templates select="$radar-config-doc/radar-config" mode="all-subsystems">
        <xsl:with-param name="dynamics-root" select="$dynamics-root"/>
        <xsl:with-param name="dynamics-configured" select="$dynamics-configured"/>
        <xsl:with-param name="packages-without-subsystem" select="$packages-without-subsystem"/>
      </xsl:apply-templates>
    </redirect:write>
    
  	<!-- create the all-packages.html at the root  --> 
    <xsl:call-template name="log-step">
     	<xsl:with-param name="log" select="$log"/>
     	<xsl:with-param name="step" select="'Generating all-packages.html'"/>
    </xsl:call-template>
   	<redirect:write file="{$docs-home}/all-packages.html">
    	<xsl:apply-templates select="Packages" mode="all-packages"/>
  	</redirect:write>
  
  	<!-- create the all-classes.html at the root  --> 
	<xsl:call-template name="log-step">
     	<xsl:with-param name="log" select="$log"/>
     	<xsl:with-param name="step" select="'Generating all-classes.html'"/>
    </xsl:call-template>
   	<redirect:write file="{$docs-home}/all-classes.html">
    	<xsl:apply-templates select="Packages" mode="all-classes"/>
  	</redirect:write>

	<!-- create the scorecard.html at the root  --> 
	<xsl:call-template name="log-step">
		<xsl:with-param name="log" select="$log"/>
		<xsl:with-param name="step" select="'Generating all-scorecard.html'"/>
	</xsl:call-template>
	<redirect:write file="{$docs-home}/scorecard.html">
		<xsl:apply-templates select="." mode="overview.scorecard"/>
	</redirect:write>
  
  	<!-- Create subsystem descriptions  --> 
    <xsl:call-template name="log-step">
     	<xsl:with-param name="log" select="$log"/>
     	<xsl:with-param name="step" select="'Generating all overview-subsystem-*.html files'"/>
    </xsl:call-template>  
    <xsl:variable name="packages-element" select="Packages"/>
      <xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
        <xsl:variable name="subsystem-name" select="name"/>
        <redirect:write file="{$docs-home}/subsystems/overview-subsystem-{normalize-space(name)}.html">
            <xsl:apply-templates select="$packages-element" mode="overview-subsystems">
                <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                <xsl:with-param name="path-to-root" select="'../'"/>
            </xsl:apply-templates>
        </redirect:write>
        <xsl:if test="$packages-without-subsystem &gt; 0">
            <xsl:call-template name="log-step">
     			<xsl:with-param name="log" select="$log"/>
     			<xsl:with-param name="step" select="'Generating all overview-subsystem-.html files (packages outside subsystem)'"/>
    		</xsl:call-template> 
	        <redirect:write file="{$docs-home}/subsystems/overview-subsystem-.html">
	            <xsl:apply-templates select="$packages-element" mode="overview-subsystems">
	                <xsl:with-param name="subsystem-name" select="''"/>
	                <xsl:with-param name="path-to-root" select="'../'"/>
	            </xsl:apply-templates>
	        </redirect:write>
        </xsl:if>
    </xsl:for-each>
    <!--  create the overview-package.html in the packages catalog -->
    <xsl:call-template name="log-step">
     	<xsl:with-param name="log" select="$log"/>
     	<xsl:with-param name="step" select="'Generating all overview-package-*.html files'"/>
    </xsl:call-template> 
    <xsl:call-template name="log-step">
     	<xsl:with-param name="log" select="$log"/>
     	<xsl:with-param name="step" select="'	- packages with no errors'"/>
    </xsl:call-template> 
	<xsl:for-each select="Packages/Package[count(error) = 0]">
        <redirect:write file="{$docs-home}/packages/overview-package-{@name}.html">
            <xsl:apply-templates select="current()" mode="overview-packages">
                <xsl:with-param name="path-to-root" select="'../'"/>
            </xsl:apply-templates>
        </redirect:write>
    </xsl:for-each>

    <xsl:call-template name="log-step">
     	<xsl:with-param name="log" select="$log"/>
     	<xsl:with-param name="step" select="'	- packages with errors'"/>
    </xsl:call-template> 
    <xsl:for-each select="Packages/Package[count(error) &gt; 0]">
        <redirect:write file="{$docs-home}/packages/overview-package-{@name}.html">
            <xsl:apply-templates select="current()" mode="error-packages">
                <xsl:with-param name="path-to-root" select="'../'"/>
            </xsl:apply-templates>
        </redirect:write>
    </xsl:for-each>

   <!-- create the classes in package view in the classes catalog -->
    <xsl:call-template name="log-step">
     	<xsl:with-param name="log" select="$log"/>
     	<xsl:with-param name="step" select="'Generating classes in package view in the classes catalog'"/>
    </xsl:call-template> 
    
    <xsl:for-each select="Packages/Package[count(error) = 0]">
        <redirect:write file="{$docs-home}/classes/{@name}-classes.html">
        <xsl:apply-templates select="current()" mode="classes-details">
               <xsl:with-param name="path-to-root" select="'../'"/>
         </xsl:apply-templates>
        </redirect:write>
    </xsl:for-each>

	<!--  create the overview-design-analysis.html at the root -->
	<xsl:call-template name="log-step">
		<xsl:with-param name="log" select="$log"/>
		<xsl:with-param name="step" select="'Generating overview-analysis.html'"/>
	</xsl:call-template> 
	<redirect:write file="{$docs-home}/overview-analysis.html">
		<xsl:apply-templates select="." mode="overview-analysis"/>
	</redirect:write>

	<!--  create the overview-explanations.html at the root -->
	<xsl:call-template name="log-step">
		<xsl:with-param name="log" select="$log"/>
		<xsl:with-param name="step" select="'Generating overview-explanations.html'"/>
	</xsl:call-template>     
	<redirect:write file="{$docs-home}/overview-explanations.html">
		<xsl:apply-templates select="." mode="explanations"/>
	</redirect:write>

</xsl:template>

  <xsl:template name="log-step">
  		<xsl:param name="log"/>
  		<xsl:param name="step"/>
  	    <xsl:choose>
    		<xsl:when test="$log = 'true'">
      			<xsl:message>
Processing step [<xsl:value-of select="$step"/>]
      			</xsl:message>
      		</xsl:when>
      	</xsl:choose>
  </xsl:template>


  <xsl:template name="index.html">
  <html>
     <head>
        <title>XRadar Statics Analysis</title>
     </head>
        <frameset cols="20%,80%">
           <frameset rows="40%,30%,30%">
              <frame src="all-subsystems.html" name="classesListFrame"/>
              <frame src="all-packages.html" name="packageListFrame"/>
              <frame src="all-classes.html" name="classesListFrame"/>
           </frameset>
           <frame src="scorecard-overview.html" name="classFrame"/>
        </frameset>
        <noframes>
           <h2>Frame Alert</h2>
           <p>
              This document is designed to be viewed using the frames feature. If you see this message, you are using a non-frame-capable web client.
           </p>
        </noframes>
  </html>
  </xsl:template>

  <xsl:template match="IntegratedAnalysis" mode="explanations">
     <xsl:variable name="metric-descriptions-doc" select="document($metric-descriptions-doc)"/>
     <html>
        <head>
           <link rel="stylesheet" type="text/css" href="stylesheet.css"/>
        </head>
        <body>
           <xsl:call-template name="page-header">
    	  	   <xsl:with-param name="main-menu-page-name" select="'explanations'"/>
             <xsl:with-param name="page-name" select="'explanations'"/>
    	  	   <xsl:with-param name="root-element" select="."/>
    	     </xsl:call-template>

            <xsl:for-each select="$metric-descriptions-doc/metric-descriptions/metric-definitions/group">
                <p>
                      <h2><xsl:value-of select="@descriptive-name"/></h2>
                      <xsl:value-of select="description"/>
                </p>
                <table class="details" width="100%">
                    <tr>
                        <th width="10%">Metric</th>
                        <th width="10%">Type</th>
                        <th width="10%">Range</th>
                        <th width="60%">Description</th>
                        <th width="10%">Source</th>
                    </tr>
                    <xsl:for-each select="metric">
                      <tr>
                        <td align="center"><xsl:value-of select="concat(@descriptive-name,' [',@abbreviation,']')"/></td>
                        <td align="center"><xsl:value-of select="@type"/></td>
                        <td align="center"><xsl:value-of select="concat(@min,' - ',@max)"/></td>
                        <td><xsl:value-of select="description"/></td>
                        <td align="center"><a href="#{@source}"><xsl:value-of select="@source"/></a></td>
                      </tr>
                    </xsl:for-each>
                </table>

                <br/>
            </xsl:for-each>
          <br/>


  		<xsl:for-each select="$metric-descriptions-doc/metric-descriptions/metric-source">
  			<br/>
  			<hr size="1"/>
  			<h2><a name="{@id}"><xsl:value-of select="name"/></a></h2>

  			<p>
  			<xsl:value-of select="description"/>
  			<br/>
  			Link : <a href="{link}"><xsl:value-of select="link"/></a>
  			</p>
  			<xsl:for-each select="metrics/metric">
  				<b><a name="{@id}" target="_new"><xsl:value-of select="name"/></a></b><br/>
  				<p>
  				<xsl:value-of select="description"/>
  				</p>
  			</xsl:for-each>
  		</xsl:for-each>

  		<hr size="1"/>
        </body>
     </html>
  </xsl:template>

  <xsl:template match="radar-config" mode="all-subsystems">
    <xsl:param name="dynamics-root"/>
    <xsl:param name="dynamics-configured"/>
    <xsl:param name="packages-without-subsystem" select="0"/>
    <xsl:choose>
	    <xsl:when test="$log = 'true'">
    		<xsl:message>
Calling radar-config with mode 'all-subsystems' with param:
dynamics-root:<xsl:value-of select="$dynamics-root"/>
dynamics-configured:<xsl:value-of select="$dynamics-configured"/>
packages-without-subsystem:<xsl:value-of select="$packages-without-subsystem"/>
    		</xsl:message>
    	</xsl:when>
    </xsl:choose>
    <html>
      <head>
        <link rel="stylesheet" type="text/css" href="stylesheet.css"/>
      </head>
      <body>


        <h2>Subsystems</h2>
          <table width="100%">
  			<tr><td nowrap="nowrap">
                <b><xsl:call-template name="system-link"/></b>
                </td></tr>
              <xsl:if test="$packages-without-subsystem &gt; 0">
              <tr><td nowrap="nowrap">
                  <b><a target="classFrame">
                  <xsl:attribute name="href"><xsl:call-template name="subsystem-url">
                                      <xsl:with-param name="subsystem-name" select="''"/>
                                    </xsl:call-template></xsl:attribute>
  	            <xsl:value-of select="'[!! There are Packages without classification - Define !!]'"/>
                  </a></b>
                </td></tr>
              </xsl:if>
              <xsl:for-each select="subsystems/subsystem">
                <tr><td nowrap="nowrap">
                <xsl:call-template name="subsystem-link">
  	            <xsl:with-param name="subsystem-name" select="normalize-space(name)"/>
                </xsl:call-template>
                </td></tr>
            </xsl:for-each>
          </table>
      </body>
    </html>
  </xsl:template>


  <xsl:template match="Packages" mode="all-packages">
    <xsl:param name="subsystem-name" select="''"/>
    <html>
      <head>
        <link rel="stylesheet" type="text/css" href="stylesheet.css"/>
      </head>
      <body>
        <h2>Packages</h2>
          <table width="100%">
  			<xsl:choose>
  				<xsl:when test="@subsystem-name != ''">
  					<xsl:apply-templates select="Package[@subsystem = $subsystem-name and count(error)=0]" mode="all.packages.link">
  					  <xsl:sort select="@name"/>
  					</xsl:apply-templates>
  					<xsl:apply-templates select="Package[@subsystem = $subsystem-name and count(error) &gt; 0]" mode="all.packages.nolink">
  					  <xsl:sort select="@name"/>
  					</xsl:apply-templates>
  				</xsl:when>
  				<xsl:otherwise>
  					<xsl:apply-templates select="Package[count(error)=0]" mode="all.packages.link">
  					  <xsl:sort select="@name"/>
  					</xsl:apply-templates>
  					<xsl:apply-templates select="Package[count(error) &gt; 0]" mode="all.packages.nolink">
  					  <xsl:sort select="@name"/>
  					</xsl:apply-templates>
  				</xsl:otherwise>
  			</xsl:choose>
          </table>
      </body>
    </html>
  </xsl:template>

  <xsl:template match="IntegratedAnalysis/Packages/Package" mode="all.packages.link">
    <tr>
      <td nowrap="nowrap">
           <a target="classFrame"><xsl:attribute name="href"><xsl:call-template name="package-url">
                                      <xsl:with-param name="package-name" select="@name"/>
                                    </xsl:call-template></xsl:attribute>
          <xsl:value-of select="@name"/>
        </a>
      </td>
    </tr>
  </xsl:template>

  <xsl:template match="IntegratedAnalysis/Packages" mode="all-classes">
    <xsl:param name="subsystem-name" select="''"/>
    <xsl:param name="package-name" select="''"/>
    <html>
      <head>
        <link rel="stylesheet" type="text/css" href="stylesheet.css"/>
      </head>
      <body>
        <h2>Classes</h2>
          <table width="100%">
  			<xsl:choose>
  				<xsl:when test="$subsystem-name != '' and $package-name = ''">
  					<xsl:apply-templates select="Package[@subsystem = $subsystem-name and count(error)=0]/child::*/Class" mode="all.classes.link">
  					  <xsl:sort select="substring-after(node(),concat(../../@name,'.'))"/>
  					</xsl:apply-templates>
  				</xsl:when>
  				<xsl:when test="$subsystem-name != '' and $package-name != ''">
  					<xsl:apply-templates select="Package[@name = $package-name and count(error)=0]/child::*/Class" mode="all.classes.link">
  					  <xsl:sort select="substring-after(node(),concat(../../@name,'.'))"/>
  					</xsl:apply-templates>
  				</xsl:when>
  				<xsl:otherwise>
  					<xsl:apply-templates select="Package[count(error)=0]/child::*/Class" mode="all.classes.link">
  					  <xsl:sort select="substring-after(node(),concat(../../@name,'.'))"/>
  					</xsl:apply-templates>
  				</xsl:otherwise>
  			</xsl:choose>
          </table>
      </body>
    </html>
  </xsl:template>

  <xsl:template match="Class" mode="all.classes.link">
    <tr>
      <td nowrap="nowrap">
        	<a target="classFrame">
  	      <xsl:attribute name="href"><xsl:call-template name="class-url">
                                      <xsl:with-param name="package-name" select="../../@name"/>
                                      <xsl:with-param name="class-name" select="normalize-space(node())"/>
                                    </xsl:call-template></xsl:attribute>
  	      <xsl:value-of select="substring-after(node(),concat(../../@name,'.'))"/>
  	    </a>
      </td>
    </tr>
  </xsl:template>

  <xsl:template match="IntegratedAnalysis/Packages/Package" mode="all.packages.nolink">
  <tr>
    <td nowrap="nowrap">
       Not Analyzed: <xsl:value-of select="@name"/>
    </td>
  </tr>
  </xsl:template>

</xsl:stylesheet>
