<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xalan="http://xml.apache.org/xalan"
	xmlns:redirect="http://xml.apache.org/xalan/redirect"
    extension-element-prefixes="redirect">

  <xsl:import href="scorecard/master-report-detail-scorecard-table.xsl"/>
  <xsl:import href="master-report-detail-classes.xsl"/>
  <xsl:import href="../../../system/radar/statics/report/packages.xsl"/>


<xsl:template match="Package" mode="overview-packages">
    <xsl:param name="path-to-root" select="'../'"/>
   <html>
      <head>
          <link rel="stylesheet" type="text/css" href="{$path-to-root}stylesheet.css"/>
      </head>
      <body>
         <xsl:call-template name="page-header">
         	<xsl:with-param name="main-menu-page-name" select="'scorecard'"/>
  	  	    <xsl:with-param name="page-name" select="'packages'"/>
            <xsl:with-param name="path-to-root" select="$path-to-root"/>
            <xsl:with-param name="package-name" select="@name"/>
  	     </xsl:call-template>

  		 <xsl:apply-templates select="current()" mode="package-summary-detail">
               <xsl:with-param name="path-to-root" select="$path-to-root"/>
         </xsl:apply-templates>
  		 <xsl:apply-templates select="current()" mode="package-summary-detail-system-specific">
               <xsl:with-param name="path-to-root" select="$path-to-root"/>
         </xsl:apply-templates>
      </body>
   </html>
</xsl:template>

<xsl:template match="Package" mode="error-packages">
    <xsl:param name="path-to-root" select="'../'"/>
   <html>
      <head>
          <link rel="stylesheet" type="text/css" href="{$path-to-root}stylesheet.css"/>
      </head>
      <body>
         <xsl:call-template name="page-header">
         	<xsl:with-param name="main-menu-page-name" select="'scorecard'"/>
  	  	    <xsl:with-param name="page-name" select="'packages'"/>
            <xsl:with-param name="path-to-root" select="$path-to-root"/>
  	     </xsl:call-template>

  		 <h3><a><xsl:attribute name="name">PK<xsl:value-of select="@name"/></xsl:attribute>
         Package : <xsl:value-of select="@name"/></a> has not been analysed.
         </h3>
 	     One of the following alternatives apply:
         <ul>
             <li>You expected this package to be analysed, but have forgotten to include it as a class file or source file to the
             XRadar. Include it and run the whole analysis again.</li>
             <li>The package is part of an external API.</li>
             <li>The package is part of the core Java API. (java.*, javax.*)</li>
         </ul>
      </body>
   </html>
</xsl:template>


<xsl:template match="Package" mode="package-summary-detail">
    <xsl:param name="path-to-root" select="''"/>
    <h3><a><xsl:attribute name="name">PK<xsl:value-of select="@name"/></xsl:attribute>
        Package : <xsl:value-of select="@name"/></a>
    </h3>

	<xsl:variable name="package-name" select="@name"/>

	<table width="100%" class="scorecard">
		<tr>
		  <td valign="top">
			 <div valign="top" align="center">
				<embed width="450" height="450" src="{$path-to-root}img/spider-graph-package-{$package-name}.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
			 </div>
			  <table width="100%" style="border : 0;">
				<tr>
				<td><b>Subsystem :
					<xsl:call-template name="subsystem-link">
						<xsl:with-param name="subsystem-name" select="subsystem/@name"/>
						<xsl:with-param name="path-to-root" select="$path-to-root"/>
					</xsl:call-template></b>
				<xsl:if test="$java-doc-configured = 'true'">
					|
					<b>
						<xsl:call-template name="package-javadoc-link">
							<xsl:with-param name="link-name" select="'Package javadoc'"/>
							<xsl:with-param name="package-name" select="@name"/>
							<xsl:with-param name="java-doc-root" select="$java-doc-root"/>
						</xsl:call-template>
					</b>
				</xsl:if>
				<xsl:if test="$jcoverage-configured = 'true'">
					|
					<b>
						<xsl:call-template name="package-jcoverage-link">
							<xsl:with-param name="link-name" select="'Package coverage details'"/>
							<xsl:with-param name="package-name" select="@name"/>
							<xsl:with-param name="jcoverage-root" select="$jcoverage-root"/>
						</xsl:call-template>
					</b>
				</xsl:if>
				<xsl:if test="$cobertura-configured = 'true'">
					|
					<b>
						<xsl:call-template name="package-cobertura-link">
							<xsl:with-param name="link-name" select="'Package coverage details'"/>
							<xsl:with-param name="package-name" select="@name"/>
							<xsl:with-param name="cobertura-root" select="$cobertura-root"/>
						</xsl:call-template>
					</b>
				</xsl:if>
					<xsl:if test="@public-api-package = 'true'">
					|<b>Package part of public system API</b>
				</xsl:if>
				</td></tr>
			</table>
		  </td>
		  <td>
			<xsl:variable name="package_quality_report">
				<xsl:value-of select="concat($docs-home,'/xml/reports/package_quality_report_',normalize-space($package-name),'.xml')"/>
			</xsl:variable>

			  <xsl:call-template name="scorecard-table">
				<xsl:with-param name="system-quality-report" 
						select="document(concat('file://',$package_quality_report))"/>
			  </xsl:call-template>
		  </td>
		</tr>
	</table>




	<!--  FIXME: XPath trick to workaround a forbidden use of current() -->
	<xsl:apply-templates select="./Stats[TotalClasses]/.." mode="package-summary-detail-header">
        <xsl:with-param name="path-to-root" select="$path-to-root"/>
    </xsl:apply-templates>

    <!--  FIXME: XPath trick to workaround a forbidden use of current() -->
    <xsl:apply-templates select="./Stats[TotalClasses]/.." mode="package-summary-detail-classes">
        <xsl:with-param name="path-to-root" select="$path-to-root"/>
    </xsl:apply-templates>

    <br/>
    <hr size="1"/>
  <br/>
</xsl:template>



<xsl:template match="Package" mode="package-summary-detail-header">
    <xsl:param name="path-to-root" select="'../'"/>
      <table width="100%" class="details">
        <tr>
          <th colspan="2">Statistics</th>
          <th colspan="2">Design</th>
          <th colspan="2">Code&#160;Quality</th>
        </tr>
        <tr>
          <td valign="top">
          	<a href="{$path-to-root}overview-explanations.html#">Total&#160;Classes</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#">Methods</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#">Source&#160;Statements&#160;[NCSS]</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#">Cyclomatic&#160;Complexity&#160;[CCN]</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#">Javadocs</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#">Javadoc&#160;Lines&#160;[JL]</a><br/>
			<br/>
          	<a href="{$path-to-root}overview-explanations.html#">Depth of inheritance tree [DIT]</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#">Number of Children [NOC]</a><br/>
			<a href="{$path-to-root}overview-explanations.html#">Efferent Class Couplings [CBO/Ce]</a><br/>
			<a href="{$path-to-root}overview-explanations.html#">Afferent Class Couplings [Ca]</a><br/>
			<a href="{$path-to-root}overview-explanations.html#">Response for a Class [RFC]</a><br/>
			<a href="{$path-to-root}overview-explanations.html#">Lack of Cohesion [LCOM]</a><br/>
          </td>
          <td valign="top">
	        <xsl:value-of select="Stats/TotalClasses"/><br/>
	      	<xsl:value-of select="Stats/functions"/><br/>
	      	<xsl:value-of select="Stats/ncss"/><br/>
            <xsl:value-of select="Stats/ccn"/><br/>
	      	<xsl:value-of select="Stats/javadocs"/><br/>
	      	<xsl:value-of select="Stats/javadoc_lines"/><br/>
			<br/>
			<xsl:value-of select="format-number(Stats/oo-metrics/dit, '#.##')"/><br/>
			<xsl:value-of select="format-number(Stats/oo-metrics/noc, '#.##')"/><br/>
			<xsl:value-of select="format-number(Stats/oo-metrics/cbo, '#.##')"/><br/>
			<xsl:value-of select="format-number(Stats/oo-metrics/ca, '#.##')"/><br/>
			<xsl:value-of select="format-number(Stats/oo-metrics/rfc, '#.##')"/><br/>
			<xsl:value-of select="format-number(Stats/oo-metrics/lcom, '#.##')"/><br/>
	      </td>

	      <td valign="top">
          	<a href="{$path-to-root}overview-explanations.html#EXafferent">Afferent&#160;Couplings</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#EXefferent">Efferent&#160;Couplings</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#EXabstractness">Abstractness</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#EXinstability">Instability</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#EXdistance">Distance&#160;[D]</a><br/>

			<xsl:if test="Stats/coupling-metrics">
			<br/>
			<a href="{$path-to-root}overview-explanations.html#">Import&#160;Couplings</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#">-&#160;Class&#160;Internal&#160;</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#">-&#160;Package&#160;Internal&#160;</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#">-&#160;Java&#160;Library&#160;</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#">Export&#160;Couplings</a><br/>
			<a href="{$path-to-root}overview-explanations.html#">-&#160;Class&#160;Internal&#160;</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#">-&#160;Package&#160;Internal&#160;</a><br/>
			</xsl:if>
          </td>
	      <td valign="top">
	        <xsl:value-of select="Stats/Ca"/><br/>
	        <xsl:value-of select="Stats/Ce"/><br/>
	        <xsl:value-of select="Stats/A"/><br/>
	        <xsl:value-of select="Stats/I"/><br/>
	        <xsl:value-of select="Stats/D"/><br/>
	        <xsl:if test="Stats/coupling-metrics">
				<br/>
				<xsl:value-of select="Stats/coupling-metrics/import-coupling"/><br/>
				<xsl:value-of select="format-number(100 * Stats/coupling-metrics/class-internal-imports div Stats/coupling-metrics/import-coupling, '#.##')"/>%<br/>
				<xsl:value-of select="format-number(100 * Stats/coupling-metrics/package-internal-imports div Stats/coupling-metrics/import-coupling, '#.##')"/>%<br/>
				<xsl:value-of select="format-number(100 * Stats/coupling-metrics/runtime-library-imports div Stats/coupling-metrics/import-coupling, '#.##')"/>%<br/>
				<xsl:value-of select="Stats/coupling-metrics/export-coupling"/><br/>
				<xsl:value-of select="format-number(100 * Stats/coupling-metrics/class-internal-exports div Stats/coupling-metrics/export-coupling, '#.##')"/>%<br/>
				<xsl:value-of select="format-number(100 * Stats/coupling-metrics/package-internal-exports div Stats/coupling-metrics/export-coupling, '#.##')"/>%<br/>
			</xsl:if>
	      </td>

          <td valign="top">
          	<a href="{$path-to-root}overview-explanations.html#">Cmplx.&#160;per&#160;Stmt.</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#">Cmplx.&#160;per&#160;Meth.</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#EXtotalviolations">Code&#160;Violations</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#EXmeanviolations">Violations&#160;per&#160;Stmt.</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#EXtotalstyleerrors">Style&#160;errors</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#EXmeanstyleerrors">St.&#160;errors&#160;per&#160;Stmt.</a><br/>
            <a href="{$path-to-root}overview-explanations.html#">Duplications</a><br/>
            <a href="{$path-to-root}overview-explanations.html#">Duplicated&#160;Tokens&#160;[DT]</a><br/>
          	<a href="{$path-to-root}overview-explanations.html#EXmeanstyleerrors">Duplicated&#160;Tokens&#160;per&#160;Stmt.</a><br/>
          </td>
          <td valign="top">

            <xsl:value-of select="format-number(Stats/ccn div Stats/ncss, '#.##')"/><br/>
            <xsl:value-of select="format-number(Stats/ccn div Stats/functions, '#.##')"/><br/>
            <xsl:value-of select="Stats/code-violations"/><br/>
            <xsl:value-of select="format-number(Stats/code-violations div Stats/ncss, '#.##')"/><br/>
            <xsl:value-of select="Stats/style-errors"/><br/>
            <xsl:value-of select="format-number(Stats/style-errors div Stats/ncss, '#.##')"/><br/>
            <xsl:value-of select="sum(Stats/duplication-metrics/duplications)"/><br/>
            <xsl:value-of select="sum(Stats/duplication-metrics/tokens)"/><br/>
            <xsl:value-of select="format-number(sum(Stats/duplication-metrics/tokens) div Stats/ncss, '#.##')"/><br/>
	      </td>
        </tr>
		<tr>
            <th colspan="2">Used by Packages</th>
			<th colspan="2">Uses Packages</th>
			<th colspan="2">Cycles</th>
		</tr>
	    <tr>
          <td colspan="2" valign="top">
	         <xsl:choose>
	            <xsl:when test="count(UsedBy/Package)=0">
	              <i>None</i>
	            </xsl:when>
	            <xsl:otherwise>
	            	<xsl:if test="count(UsedBy/Package[@legal = 'true']) &gt; 0">
		            	<b>Legal Use:</b><br/>
			            <xsl:for-each select="UsedBy/Package[@legal = 'true']">
			              <a>
                            <xsl:attribute name="href"><xsl:call-template name="package-url">
                                    <xsl:with-param name="package-name" select="normalize-space(node())"/>
                                    <xsl:with-param name="path-to-root" select="$path-to-root"/>
                                  </xsl:call-template></xsl:attribute>
			                <xsl:value-of select="node()"/>
			              </a><br/>
			            </xsl:for-each>
		            </xsl:if>
		            <xsl:if test="count(UsedBy/Package[@legal != 'true']) &gt; 0">
			            <b>Illegal Use:</b><br/>
			            <xsl:for-each select="UsedBy/Package[@legal!='true']">
			              <a>
			                <xsl:attribute name="href"><xsl:call-template name="package-url">
                                    <xsl:with-param name="package-name" select="normalize-space(node())"/>
                                    <xsl:with-param name="path-to-root" select="$path-to-root"/>
                                  </xsl:call-template></xsl:attribute>
			                <xsl:value-of select="node()"/>
			              </a><br/>
			            </xsl:for-each>
		            </xsl:if>
		        </xsl:otherwise>
		     </xsl:choose>
          </td>

          <td colspan="2" valign="top">
            <xsl:choose>
	            <xsl:when test="count(DependsUpon/Package)=0">
	              <i>None</i>
	            </xsl:when>
	            <xsl:otherwise>
	            	<xsl:if test="count(DependsUpon/Package[@legal = 'true']) &gt; 0">
			            <b>Legal Dependencies:</b><br/>
			            <xsl:for-each select="DependsUpon/Package[@legal='true']">
			              <a>
			                <xsl:attribute name="href"><xsl:call-template name="package-url">
                                    <xsl:with-param name="package-name" select="normalize-space(node())"/>
                                    <xsl:with-param name="path-to-root" select="$path-to-root"/>
                                  </xsl:call-template></xsl:attribute>
			                <xsl:value-of select="node()"/>
			              </a><br/>
			            </xsl:for-each>
		            </xsl:if>
		            <xsl:if test="count(DependsUpon/Package[(@legal != 'true') and not(@external='true')]) &gt; 0">
			            <b>Illegal Dependencies:</b><br/>
			            <xsl:for-each select="DependsUpon/Package[(@legal !='true') and not(@external='true')]">
			              <a>
			                <xsl:attribute name="href"><xsl:call-template name="package-url">
                                    <xsl:with-param name="package-name" select="normalize-space(node())"/>
                                    <xsl:with-param name="path-to-root" select="$path-to-root"/>
                                  </xsl:call-template></xsl:attribute>
			                <xsl:value-of select="node()"/>
			              </a><br/>
			            </xsl:for-each>
			        </xsl:if>

			        <xsl:if test="count(DependsUpon/Package[@external = 'true']) &gt; 0">
			            <b>External Dependencies:</b><br/>
			            <xsl:for-each select="DependsUpon/Package[@external='true']">
			              <a>
			                <xsl:attribute name="href"><xsl:call-template name="package-url">
                                    <xsl:with-param name="package-name" select="normalize-space(node())"/>
                                    <xsl:with-param name="path-to-root" select="$path-to-root"/>
                                  </xsl:call-template></xsl:attribute>
			                <xsl:value-of select="node()"/>
			              </a><br/>
			            </xsl:for-each>
			        </xsl:if>
		        </xsl:otherwise>
	        </xsl:choose>
          </td>

          <td colspan="2" valign="top">
	          <xsl:if test="Cycles">
			    <xsl:for-each select="Cycles/Package">
			      <a>
			      <xsl:attribute name="href"><xsl:call-template name="package-url">
                                    <xsl:with-param name="package-name" select="normalize-space(node())"/>
                                    <xsl:with-param name="path-to-root" select="$path-to-root"/>
                                  </xsl:call-template></xsl:attribute>
			        <xsl:value-of select="node()"/>
			      </a><br/>
			    </xsl:for-each>
			  </xsl:if>
              <xsl:if test="not(Cycles)">
                <i>None</i>
              </xsl:if>
          </td>
        </tr>
        <tr>
            <th colspan="2">Tests</th>
            <th colspan="2">Source Control</th>
        </tr>
        <tr>
            <xsl:if test="Stats/test-metrics">
                <td valign="top">
                    <a href="{$path-to-root}overview-explanations.html#">Test&#160;Suites</a><br/>
                    <a href="{$path-to-root}overview-explanations.html#">Tests</a><br/>
                    <a href="{$path-to-root}overview-explanations.html#">Source&#160;Statements&#160;per&#160;Tests</a><br/>
                    <a href="{$path-to-root}overview-explanations.html#">Errors</a><br/>
                    <a href="{$path-to-root}overview-explanations.html#">Time</a><br/>
                    <a href="{$path-to-root}overview-explanations.html#">Line&#160;Coverage</a><br/>
                    <a href="{$path-to-root}overview-explanations.html#">Line&#160;Coverage&#160;per&#160;Test</a><br/>
                </td>
                <td valign="top">

                  <xsl:variable name="suites" select="sum(Stats/test-metrics/testsuites)"/>
                  <xsl:variable name="tests" select="sum(Stats/test-metrics/tests)"/>
                  <xsl:variable name="errors" select="sum(Stats/test-metrics/errors)"/>
                  <xsl:variable name="time" select="sum(Stats/test-metrics/time)"/>
                  <xsl:variable name="line-coverage" select="sum(Stats/test-metrics/line-coverage)"/>
                  <xsl:variable name="source-statements" select="Stats/ncss"/>


                  <xsl:value-of select="$suites"/><br/>
                  <xsl:value-of select="$tests"/><br/>
                  <xsl:value-of select="format-number($source-statements div $tests, '#.##')"/><br/>
                  <xsl:value-of select="$errors"/><br/>
                  <xsl:value-of select="format-number($time, '#.##')"/><br/>
                  <xsl:value-of select="format-number(100 * $line-coverage, '#.##')"/>%<br/>
                  <xsl:value-of select="format-number(100 * $line-coverage div $tests, '#.##')"/>%<br/>
                </td>
            </xsl:if>
            <xsl:if test="not(Stats/test-metrics)">
                <td valign="top" colspan="2">
                    <i>None</i>
                </td>
            </xsl:if>
            <xsl:if test="Stats/source-control-metrics">
                <xsl:variable name="package-changes" select="sum(Stats/source-control-metrics/change-activity)"/>
                <xsl:variable name="package-errors" select="sum(Stats/source-control-metrics/error-activity)"/>
                <xsl:variable name="package-total" select="$package-changes + $package-errors"/>

                <xsl:variable name="subsystem" select="subsystem/@name"/>
                <xsl:variable name="subsystem-changes" select="sum(../Package[subsystem/@name=$subsystem]/Stats/source-control-metrics/change-activity)"/>
                <xsl:variable name="subsystem-errors" select="sum(../Package[subsystem/@name=$subsystem]/Stats/source-control-metrics/error-activity)"/>
                <xsl:variable name="subsystem-total" select="$subsystem-changes + $subsystem-errors"/>

                <td valign="top">
                    Total<br/>
                    Changes<br/>
                    Errors<br/>
                    <b>Percentage in module:</b><br/>
                    Total<br/>
                    Changes<br/>
                    Errors<br/>
                </td>
                <td valign="top">
                  <xsl:value-of select="$package-total"/><br/>
                  <xsl:value-of select="$package-changes"/><br/>
                  <xsl:value-of select="$package-errors"/><br/>
                  <br/>
                  <xsl:value-of select="format-number(100 * $package-total div $subsystem-total, '#.##')"/><br/>
                  <xsl:value-of select="format-number(100 * $package-changes div $subsystem-changes, '#.##')"/><br/>
                  <xsl:value-of select="format-number(100 * $package-errors div $subsystem-errors, '#.##')"/><br/>
                </td>
            </xsl:if>
            <xsl:if test="not(Stats/source-control-metrics)">
                <td valign="top" colspan="2">
                    <i>None</i>
                </td>
            </xsl:if>
        </tr>
      </table>
      <br/>
</xsl:template>




<xsl:template match="Package" mode="package-summary-detail-classes">
    <xsl:param name="path-to-root" select="'../'"/>
	<table width="100%" class="details">
        <tr>
			<th>Abstract Class</th>
			<th>Source statements</th>
			<th>Cyclometic complexity</th>
			<th>Violations</th>
			<th>Style Errors</th>
			<th>Duplications</th>
			<th>Efferent (Import) Class Couplings (CBO / Ce)</th>
			<th>Afferent (Emport) Class Couplings (Ca)</th>
			<th>Response for Class (RFC)</th>
			<th>Lack of Cohesion in Methods (LCOM)</th>
			<th>Depth of Inheritance Rree (DIT)</th>
			<th>Statement Test Coverage</th>
			<th>Method Test Reference</th>
			<th>Illegal Imports</th>
		</tr>
        <xsl:if test="count(AbstractClasses/Class)=0">
          <tr>
          <td colspan="20" align="left"><i>None</i></td>
          </tr>
        </xsl:if>
        <xsl:apply-templates select="AbstractClasses/Class" mode="standard-class-row">
			<xsl:with-param name="skip-package-name" select="'true'"/>
		</xsl:apply-templates>
      </table>

      <br/>

      <table width="100%" class="details">
        <tr>
			<th>Concrete Class</th>
			<th>Source statements</th>
			<th>Cyclometic complexity</th>
			<th>Violations</th>
			<th>Style Errors</th>
			<th>Duplications</th>
			<th>Efferent (Import) Class Couplings (CBO / Ce)</th>
			<th>Afferent (Emport) Class Couplings (Ca)</th>
			<th>Response for Class (RFC)</th>
			<th>Lack of Cohesion in Methods (LCOM)</th>
			<th>Depth of Inheritance Rree (DIT)</th>
			<th>Statement Test Coverage</th>
			<th>Method Test Reference</th>
			<th>Illegal Imports</th>
		</tr>
        <xsl:if test="count(ConcreteClasses/Class)=0">
          <tr>
          <td colspan="20" align="left"><i>None</i></td>
          </tr>
        </xsl:if>
		<xsl:apply-templates select="ConcreteClasses/Class" mode="standard-class-row">
			<xsl:with-param name="skip-package-name" select="'true'"/>
		</xsl:apply-templates>
      </table>
      <br/>
</xsl:template>

</xsl:stylesheet>
